package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.knpcv1.mode.PressureMode;
import jakarta.validation.constraints.NotNull;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * 逻辑定值-简单模式定值数据寄存器地址
 *
 * @author <a href="mailto:scwsl@foxmail.com>王仕龙</a>
 * 2024/11/28 0:31
 */
@Getter
@Setter
@SuperBuilder
public class SimpleLogicMode extends Command {
    @Serial
    private static final long serialVersionUID = 8899565585137994751L;
    /**
     * 开井时间，以秒为单位的时间
     */
    private int wellOpenTimeTimestamp;
    /**
     * 开井套压
     */
    @NotNull(message = "开井套压不能为空")
    private BigDecimal wellOpenPressureValue = BigDecimal.ZERO;
    /**
     * 开井油压
     */
    @NotNull(message = "开井油压不能为空")
    private BigDecimal wellOpenOilPressureValue = BigDecimal.ZERO;
    /**
     * 开井载荷因子预设值
     */
    @NotNull(message = "开井载荷因子预设值不能为空")
    private BigDecimal wellOpenLoadFactorPresets = BigDecimal.ZERO;
    /**
     * 关井时间
     */
    private int wellCloseTimeTimestamp;
    /**
     * 关井套压
     */
    private int wellClosePressureValue;
    /**
     * 关井油压
     */
    private int wellCloseOilPressureValue;
    /**
     * 关井流量
     */
    private int wellCloseFlowValue;
    /**
     * 最小开井时间
     */
    private int minWellOpenTimeDuration;
    /**
     * 最大开井时间
     */
    private int maxWellOpenTimeDuration;
    /**
     * 最小关井时间
     */
    private int minWellCloseTimeDuration;
    /**
     * 最大关井时间
     */
    private int maxWellCloseTimeDuration;
    /**
     * 压力稳定时间
     */
    private int pressureStabilizationDuration;
    /**
     * 流量稳定时间
     */
    private int flowStabilizationDuration;
    /**
     * 载荷因子稳定时间
     */
    private int loadFactorStabilizationDuration;
    /**
     * 柱塞延迟时间
     */
    private int plungerDelayDuration;

    public SimpleLogicMode() {
        this.setCode("SCSS.SIMPLE_LOGIC_MODE");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && Objects.nonNull(this.wellOpenPressureValue)
                && this.wellOpenPressureValue.compareTo(BigDecimal.ZERO) >= 0
                && this.wellOpenPressureValue.multiply(PressureMode.MULTIPLE).compareTo(PressureMode.MAX_VALUE) <= 0
                && Objects.nonNull(this.wellOpenOilPressureValue)
                && this.wellOpenOilPressureValue.compareTo(BigDecimal.ZERO) >= 0
                && this.wellOpenOilPressureValue.multiply(PressureMode.MULTIPLE).compareTo(PressureMode.MAX_VALUE) <= 0
                && Objects.nonNull(this.wellOpenLoadFactorPresets)
                && this.wellOpenLoadFactorPresets.compareTo(BigDecimal.ZERO) >= 0
                && this.wellOpenLoadFactorPresets.multiply(PressureMode.MULTIPLE).compareTo(PressureMode.MAX_VALUE) <= 0;
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(130);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0096    0020     40
        command.append("01100096002040");
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenTimeTimestamp), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenPressureValue.multiply(PressureMode.MULTIPLE).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenOilPressureValue.multiply(PressureMode.MULTIPLE).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenLoadFactorPresets.multiply(PressureMode.MULTIPLE).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseTimeTimestamp), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellClosePressureValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseOilPressureValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseFlowValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.minWellOpenTimeDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.maxWellOpenTimeDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.minWellCloseTimeDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.maxWellCloseTimeDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureStabilizationDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowStabilizationDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.loadFactorStabilizationDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.plungerDelayDuration), 8, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }
}
